#!/usr/bin/env python3

import fire
import logging
import qsAPI
import qsense

class Qsense(object):
    def export_apps(self, host, certificate, targetPath, pFilter="stream.name ne 'None'"):
        """Export applications to files"""
        qrs = qsAPI.QRS(proxy=host, certificate=certificate)
        qsense.apps.export(qrs, targetPath=targetPath, pFilter=pFilter)

    def export_users(self, host, certificate):
        """Export users and his/her groups"""
        qrs = qsAPI.QRS(proxy=host, certificate=certificate)
        qsense.users.export_users_and_groups(qrs)

    def export_delete_old_apps(self, host, certificate, target_path,
                               modified_days=180,
                               last_reload_days=180,
                               published=False,
                               save_meta=True,
                               skipdata=True,
                               export=True,
                               delete=False):
        """Export and delete old apps using 'modified_date' and 'last_reload_time' filters."""
        qrs = qsAPI.QRS(proxy=host, certificate=certificate)
        return qsense.apps.export_delete_old_apps(qrs, target_path, modified_days, last_reload_days, published=published, save_meta=save_meta, skipdata=skipdata, export=export, delete=delete)

def main():
    logging.basicConfig(level=logging.DEBUG)
    fire.Fire(Qsense)

if __name__ == "__main__":
    main()



import qsAPI
import requests

def engine(host, scheme, port, timeout, vproxy="", headers=[]):
    ##  https://help.qlik.com/en-US/sense-developer/February2020/Subsystems/EngineAPI/Content/Sense_EngineAPI/GettingSystemInformation/HealthCheckStatus.htm
    ## TODO authentication
    path="/{vproxy}/engine/healthcheck/".format(vproxy=vproxy)
    url = "{scheme}://{host}:{port}{path}".format(scheme=scheme, host=host, port=port)
    get_and_return_status(url, timeout)

def get_and_return_status(url, timeout):
    r = requests.get(url, timeout=timeout)
    return r.status_code

def hub(url, timeout):
    get_and_return_status(url, timeout)

def proxy(host, scheme, port, timeout):
    ## https://help.qlik.com/en-US/sense-developer/February2020/Subsystems/ProxyServiceAPI/Content/Sense_ProxyServiceAPI/ProxyServiceAPI-Introduction.htm
    path="/qps/user"
    url = "{scheme}://{host}:{port}{path}".format(scheme=scheme, host=host, port=port)
    get_and_return_status(url, timeout)


def repository_service(host, port, certificate):
    ## qps = ...
    "TODO"



import qsAPI
import re

def delete_removed_exernally_users(qrs, user_directory, dryrun=True):
        users = qrs.UserGet(pFilter="userDirectory eq '%s' and removedExternally eq True" % user_directory)
        logging.warning("Users to be deleted: %s" % len(users))
        for u in users:
              loggin.warning("User to be deleted: %s" % u["name"])
              if not dryrun:
                    qrs.UserDelete(u['id'])

def export_users_and_groups(qrs, pFilter="removedExternally ne True", pUserID="full", groupFilter="^QLIKSENSE_", userAttribute="name", sep="\t"):
    users = qrs.UserGet(pFilter=pFilter, pUserID=pUserID)
    for u in users:
        if "attributes" in u.keys():
            for a in u["attributes"]:
                groupname = a["attributeValue"]
                if re.match(groupFilter, groupname, re.IGNORECASE):
                    print("{name}{sep}{groupname}\n".format(name=u[userAttribute],
                                                              groupname=groupname,
                                                              sep=sep))


def user_sessions(hosts, certificate, usergroup, userid, vproxies=""):
    for server in hosts.split(","):
        for vproxy in vproxies.split(","):
            qps=qsAPI.QPS(proxy=server, certificate=certificate)
            print(server + " : " + vproxy)
            print(qps.GetUser(usergroup, user).json())



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="qsense",
    version="0.2.0",
    author="Matteo Redaelli",
    author_email="matteo.redaelli@gmail.com",
    description="qsense is a useful library and command line tool for qliksense",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/matteoredaelli/qsense",
    packages=setuptools.find_packages(),
    license="GPL",
    entry_points = {
        'console_scripts': ['qsense=qsense.command_line:main'],
    },

    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests', 'requests_ntlm', 'qsapi', 'fire'],
    python_requires='>=3.6',
)



import qsAPI
import re
import logging

def deallocate_analyzer_licenses_for_professionals(qrs, dryrun=True):
    def extract_users(licenses):
        return list(map(lambda u: u["user"]["userId"], licenses))
    def extract_licenses_for_user(licenses, userId):
        return list(filter(lambda u: u["user"]["userId"] == userId, licenses))

    analyzers = qrs.driver.get('/qrs/license/analyzeraccesstype/full').json()
    logging.debug("Analyzer licenses: %d" % len(analyzers))
    analyzers_list = extract_users(analyzers)

    professionals = qrs.driver.get('/qrs/license/professionalaccesstype/full').json()
    logging.debug("Professional licenses: %d" % len(professionals))
    professionals_list = extract_users(professionals)

    to_be_removes = list(set(analyzers_list) & set(professionals_list) )
    logging.warning("Licenses to be removed: %d" % len(to_be_removes))

    for u in list(map(lambda u: extract_licenses_for_user(analyzers, u), to_be_removes)):
        assert len(u) == 1
        u = u[0]
        logging.warning("removing analyzer license for user %s" % u["user"]["name"])
        if not dryrun:
            qrs.driver.delete('/qrs/license/analyzeraccesstype/' + u["id"])



import qsAPI
import logging
import json
import os
import re
from datetime import date
from datetime import timedelta

def export_apps(qrs, target_path, apps, save_meta=True, skipdata=True):
    for app in apps:
        logging.info("Exporting app: " + str(app))
        filename = app['modifiedByUserName'] + "-" + app['name'] + "-" + app["id"]
        filename = re.sub("[\\\/& .]", '_', filename)
        filename = os.path.join(target_path, filename)
        if save_meta:
            with open(filename + ".json", "w") as f:
                f.write(json.dumps(apps, indent=4))
        qrs.AppExport(app['id'],  filename + ".qvd", skipdata=skipdata)

def export_by_filter(qrs, target_path, pFilter="stream.name ne 'None'", save_meta=True, skipdata=True):
    apps = qrs.AppGet(pFilter=pFilter)
    return export_apps(qrs, target_path=target_path, apps=apps, save_meta=save_meta, skipdata=skipdata)

def get_old_apps(qrs, modified_days=180, last_reload_days=180, published=False):
    today = date.today()
    modified_date = (today - timedelta(days=modified_days)).strftime("%Y-%m-%d %H:%M:%S")
    logging.debug("Modified date = " + modified_date)

    last_reload_time = (today - timedelta(days=last_reload_days)).strftime("%Y-%m-%d %H:%M:%S")
    logging.debug("Last reload time = " + last_reload_time)
    pFilter="published eq {published} and modifiedDate lt '{modified}' and lastReloadTime lt '{reload}'".format(published=published,
                                                                                                                modified=modified_date,
                                                                                                                reload=last_reload_time)
    logging.debug("Search apps with pFilter= " + str(pFilter))
    apps = qrs.AppGet(pFilter=pFilter)
    ##l = map(lambda a: { key:value for key,value in a.items()
    ##                    if key in ['id', 'name',
    ##                               'createdDate',
    ##                               'modifiedDate', 'modifiedByUserName',
    ##                               'published', 'lastReloadTime', 'fileSize']}, apps)
    return apps


def export_delete_old_apps(qrs, target_path, modified_days, last_reload_days, published=False, save_meta=True, skipdata=True, export=True, delete=False):
    if delete and (modified_days < 60 or last_reload_days < 60):
        logginf.error("You want to delete too recent apps. Bye")
        return 1
    apps = get_old_apps(qrs, modified_days, last_reload_days, published)
    if export:
        export_apps(qrs, apps=apps, target_path=target_path, save_meta=save_meta, skipdata=skipdata)
    if delete:
        logging.warning("Removing app: " + str(app))
        qrs.AppDelete(app['id'])
